<?php

namespace Drupal\openlayers\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;

/**
 * Plugin implementation of the 'openlayers_default' formatter.
 *
 * @FieldFormatter(
 *   id = "openlayers_formatter_default",
 *   label = @Translation("OpenLayers"),
 *   field_types = {
 *     "geofield"
 *   }
 * )
 */
class OpenLayersDefaultFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'openlayers_map' => 'none selected',
      'mapeid' => 'blabla',
      'mapheight' => 450,
      'zoom' => 9,
      'minzoom' => 1,
      'maxzoom' => 18,
    ] + parent::defaultSettings();
  }

  /**
   * Get the view settings of a map.
   */
  public function getMapSettings($form_state) {
    $mapid = $form_state['fields']['#value']['field_geofield']['settings_edit_form']['settings']['openlayers_map'];
    $entity_map = \Drupal::service('entity.repository')->loadEntityByUuid('openlayers_map', $mapid);

    $mapheight = $entity_map->map_height->value;
    $zoom = $entity_map->zoom->value;
    $minzoom = $entity_map->minzoom->value;
    $maxzoom = $entity_map->maxzoom->value;

    $response = new AjaxResponse();
    $response->addCommand(new InvokeCommand('#openlayers-map-zoom', 'val', [$zoom]));
    $response->addCommand(new InvokeCommand('#openlayers-map-minzoom', 'val', [$minzoom]));
    $response->addCommand(new InvokeCommand('#openlayers-map-maxzoom', 'val', [$maxzoom]));
    $response->addCommand(new InvokeCommand('#openlayers-map-mapheight', 'val', [$mapheight]));
    $response->addCommand(new InvokeCommand('#openlayers-map-mapid', 'val', [$mapid]));

    return $response;
  }

  /**
   * {@inheritdoc}
   */

  /**
   * Formular bei der Auswahl der DisplaySettings.
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $openlayers_map_options = [];
    foreach (openlayers_map_get_info() as $key => $map) {
      $openlayers_map_options[$key] = $this->t($map['label']);
    }

    $elements['openlayers_map'] = [
      '#title' => $this->t('OpenLayers Map'),
      '#type' => 'select',
      '#options' => $openlayers_map_options,
      '#default_value' => $this->getSetting('openlayers_map'),
      '#required' => TRUE,
      '#attributes' => ['class' => ['openlayers-map-selector']],
      '#ajax' => [
        'callback' => 'Drupal\openlayers\Plugin\Field\FieldFormatter\OpenLayersDefaultFormatter::getMapSettings',
        'event' => 'change',
        'progress' => [
          'type' => 'throbber',
          'message' => NULL,
        ],
      ],
    ];
    $elements['zoom'] = [
      '#title' => $this->t('Zoom'),
      '#type' => 'number',
      '#attributes' => ['id' => ['openlayers-map-zoom']],
      '#default_value' => $this->getSetting('zoom'),
      '#required' => TRUE,
    ];
    $elements['minzoom'] = [
      '#title' => $this->t('Min. Zoom'),
      '#type' => 'number',
      '#attributes' => ['id' => ['openlayers-map-minzoom']],
      '#default_value' => $this->getSetting('minzoom'),
      '#required' => TRUE,
    ];
    $elements['maxzoom'] = [
      '#title' => $this->t('Max. Zoom'),
      '#type' => 'number',
      '#attributes' => ['id' => ['openlayers-map-maxzoom']],
      '#default_value' => $this->getSetting('maxzoom'),
      '#required' => TRUE,
    ];
    $elements['mapheight'] = [
      '#title' => $this->t('Map Height'),
      '#type' => 'number',
      '#attributes' => ['id' => ['openlayers-map-mapheight']],
      '#default_value' => $this->getSetting('mapheight'),
      '#field_suffix' => $this->t('px'),
    ];
    $elements['mapeid'] = [
      '#title' => $this->t('Map Entity ID'),
      '#type' => 'textfield',
      '#size' => 36,
      '#disabled' => TRUE,
      '#attributes' => ['id' => ['openlayers-map-mapid']],
      '#default_value' => $this->getSetting('openlayers_map'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = $this->t('OpenLayers MAP: @map', ['@map' => $this->getSetting('openlayers_map')]);
    $summary[] = $this->t('Map height: @height px', ['@height' => $this->getSetting('mapheight')]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   *
   * This function is called from parent::view().
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $settings = $this->getSettings();
    $map = openlayers_map_get_info($settings['openlayers_map']);
    $map['settings']['zoom'] = isset($settings['zoom']) ? $settings['zoom'] : NULL;
    $map['settings']['minzoom'] = isset($settings['minzoom']) ? $settings['minzoom'] : NULL;
    $map['settings']['maxzoom'] = isset($settings['maxzoom']) ? $settings['maxzoom'] : NULL;

    $elements = [];
    foreach ($items as $delta => $item) {
      $features = openlayers_process_geofield($item->value);
      /*
       * Folgende Zeilen bin ich mir noch nicht sicher, was diese tun, hab sie mal auskommentiert.
       */
      $elements[$delta] = openlayers_render_map($map, $features, $settings['mapheight'] . 'px');
    }
    return $elements;
  }

  /**
   * Validate Url method.
   *
   * @param array $element
   *   The element to validate.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The state of the form.
   */
  public function validateUrl($element, FormStateInterface $form_state) {
    if (!empty($element['#value']) && !UrlHelper::isValid($element['#value'])) {
      $form_state->setError($element, $this->t("Icon Url is not valid."));
    }
  }

}
